@@ -14,35 +14,88 @@ methods returning instances of ``QIcon``.
 """
 
 # Standard library imports
-from __future__ import print_function
-import hashlib
+import ctypes
+import filecmp
 import json
 import os
+import shutil
 import warnings
 
 # Third party imports
-from qtpy.QtCore import QByteArray, QObject, QPoint, QRect, Qt
+from qtpy.QtCore import (QByteArray, QObject, QPoint, QRect, Qt,
+                         QSizeF, QRectF, QPointF, QThread)
 from qtpy.QtGui import (QColor, QFont, QFontDatabase, QIcon, QIconEngine,
-                        QPainter, QPixmap, QTransform)
+                        QPainter, QPixmap, QTransform, QPalette, QRawFont,
+                        QImage)
 from qtpy.QtWidgets import QApplication
 
+try:
+    # Needed since `QGlyphRun` is not available for PySide2
+    # See spyder-ide/qtawesome#210
+    from qtpy.QtGui import QGlyphRun
+except ImportError:
+    QGlyphRun = None
+
 # Linux packagers, please set this to True if you want to make qtawesome
 # use system fonts
 SYSTEM_FONTS = False
 
-# MD5 Hashes for font files bundled with qtawesome:
-MD5_HASHES = {
-    'fontawesome4.7-webfont.ttf': 'b06871f281fee6b241d60582ae9369b9',
-    'fontawesome5-regular-webfont.ttf': '6a745dc6a0871f350b0219f5a2678838',
-    'fontawesome5-solid-webfont.ttf': 'acf50f59802f20d8b45220eaae532a1c',
-    'fontawesome5-brands-webfont.ttf': 'ed2b8bf117160466ba6220a8f1da54a4',
-    'elusiveicons-webfont.ttf': '207966b04c032d5b873fd595a211582e',
-    'materialdesignicons-webfont.ttf': 'b0fd91bb29dcb296a9a37f8bda0a2d85',
-}
+# Needed imports and constants to install bundled fonts on Windows
+# Based on https://stackoverflow.com/a/41841088/15954282
+if os.name == 'nt':
+    from ctypes import wintypes
+    import winreg
+    
+    user32 = ctypes.WinDLL('user32', use_last_error=True)
+    gdi32 = ctypes.WinDLL('gdi32', use_last_error=True)
+
+    FONTS_REG_PATH = r'Software\Microsoft\Windows NT\CurrentVersion\Fonts'
+    GFRI_DESCRIPTION = 1
+    GFRI_ISTRUETYPE  = 3
+
+    if not hasattr(wintypes, 'LPDWORD'):
+        wintypes.LPDWORD = ctypes.POINTER(wintypes.DWORD)
+
+    user32.SendMessageTimeoutW.restype = wintypes.LPVOID
+    user32.SendMessageTimeoutW.argtypes = (
+        wintypes.HWND,   # hWnd
+        wintypes.UINT,   # Msg
+        wintypes.LPVOID, # wParam
+        wintypes.LPVOID, # lParam
+        wintypes.UINT,   # fuFlags
+        wintypes.UINT,   # uTimeout
+        wintypes.LPVOID) # lpdwResult
+
+    gdi32.AddFontResourceW.argtypes = (
+        wintypes.LPCWSTR,) # lpszFilename
+
+    # http://www.undocprint.org/winspool/getfontresourceinfo
+    gdi32.GetFontResourceInfoW.argtypes = (
+        wintypes.LPCWSTR, # lpszFilename
+        wintypes.LPDWORD, # cbBuffer
+        wintypes.LPVOID,  # lpBuffer
+        wintypes.DWORD)   # dwQueryType
+
+
+def text_color():
+    try:
+        palette = QApplication.instance().palette()
+        return palette.color(QPalette.Active, QPalette.Text)
+    except AttributeError:
+        return QColor(50, 50, 50)
+
+
+def text_color_disabled():
+    try:
+        palette = QApplication.instance().palette()
+        return palette.color(QPalette.Disabled, QPalette.Text)
+    except AttributeError:
+        return QColor(150, 150, 150)
+
 
 _default_options = {
-    'color': QColor(50, 50, 50),
-    'color_disabled': QColor(150, 150, 150),
+    'color': text_color,
+    'color_disabled': text_color_disabled,
     'opacity': 1.0,
     'scale_factor': 1.0,
 }
@@ -59,7 +112,8 @@ def set_global_defaults(**kwargs):
         'color_active', 'color_selected', 'color_disabled',
         'color_on_selected', 'color_on_active', 'color_on_disabled',
         'color_off_selected', 'color_off_active', 'color_off_disabled',
-        'animation', 'offset', 'scale_factor', 'rotated', 'hflip', 'vflip'
+        'animation', 'offset', 'scale_factor', 'rotated', 'hflip', 'vflip',
+        'draw'
         ]
 
     for kw in kwargs:
@@ -82,8 +136,6 @@ class CharIconPainter:
     def _paint_icon(self, iconic, painter, rect, mode, state, options):
         """Paint a single icon."""
         painter.save()
-        color = options['color']
-        char = options['char']
 
         color_options = {
             QIcon.On: {
@@ -108,8 +160,18 @@ class CharIconPainter:
         }
 
         color, char = color_options[state][mode]
+        alpha = None
 
-        painter.setPen(QColor(color))
+        # If color comes as a tuple, it means we need to set alpha on it.
+        if isinstance(color, tuple):
+            alpha = color[1]
+            color = color[0]
+
+        qcolor = QColor(color)
+        if alpha:
+            qcolor.setAlpha(alpha)
+
+        painter.setPen(qcolor)
 
         # A 16 pixel-high icon yields a font size of 14, which is pixel perfect
         # for font-awesome. 16 * 0.875 = 14
@@ -124,40 +186,98 @@ class CharIconPainter:
         if animation is not None:
             animation.setup(self, painter, rect)
 
-        painter.setFont(iconic.font(prefix, draw_size))
         if 'offset' in options:
             rect = QRect(rect)
             rect.translate(round(options['offset'][0] * rect.width()),
                            round(options['offset'][1] * rect.height()))
 
-        if 'vflip' in options and options['vflip'] == True:
-            x_center = rect.width() * 0.5
-            y_center = rect.height() * 0.5
-            painter.translate(x_center, y_center)
-            transfrom = QTransform()
-            transfrom.scale(1,-1)
-            painter.setTransform(transfrom, True)
-            painter.translate(-x_center, -y_center)
-
-        if 'hflip' in options and options['hflip'] == True:
-            x_center = rect.width() * 0.5
-            y_center = rect.height() * 0.5
-            painter.translate(x_center, y_center)
-            transfrom = QTransform()
-            transfrom.scale(-1, 1)
-            painter.setTransform(transfrom, True)
-            painter.translate(-x_center, -y_center)
-
+        x_center = rect.width() * 0.5
+        y_center = rect.height() * 0.5
+        transform = QTransform()
+        transform.translate(+x_center, +y_center)
+        if 'vflip' in options and options['vflip'] is True:
+            transform.scale(1,-1)
+        if 'hflip' in options and options['hflip'] is True:
+            transform.scale(-1, 1)
         if 'rotated' in options:
-            x_center = rect.width() * 0.5
-            y_center = rect.height() * 0.5
-            painter.translate(x_center, y_center)
-            painter.rotate(options['rotated'])
-            painter.translate(-x_center, -y_center)
+            transform.rotate(options['rotated'])
+        transform.translate(-x_center, -y_center)
+        painter.setTransform(transform, True)
 
         painter.setOpacity(options.get('opacity', 1.0))
 
-        painter.drawText(rect, int(Qt.AlignCenter | Qt.AlignVCenter), char)
+        draw = options.get('draw')
+        if draw not in ('text', 'path', 'glyphrun', 'image'):
+            # Use QPainterPath when setting an animation
+            # to fix tremulous spinning icons.
+            # See spyder-ide/qtawesome#39
+            draw = 'path' if animation is not None else 'text'
+
+        def try_draw_rawfont():
+            if draw == 'glyphrun' and animation is not None:
+                # Disable font hinting to mitigate tremulous spinning to some extent
+                # See spyder-ide/qtawesome#39
+                rawfont = iconic.rawfont(prefix, draw_size, QFont.PreferNoHinting)
+            else:
+                rawfont = iconic.rawfont(prefix, draw_size)
+
+            # Check glyf table and fallback to draw text if missing
+            # because font glyph is necessary to draw path/glyphrun/image.
+            if not rawfont.fontTable('glyf'):
+                return False
+
+            glyph = rawfont.glyphIndexesForString(char)[0]
+            advance = rawfont.advancesForGlyphIndexes((glyph,))[0]
+            ascent = rawfont.ascent()
+            size = QSizeF(abs(advance.x()), ascent + rawfont.descent())
+            painter.translate(QRectF(rect).center())
+            painter.translate(-size.width() / 2, -size.height() / 2)
+
+            if draw == 'path':
+                path = rawfont.pathForGlyph(glyph)
+                path.translate(0, ascent)
+                path.setFillRule(Qt.WindingFill)
+                painter.setRenderHint(QPainter.Antialiasing, True)
+                painter.fillPath(path, painter.pen().color())
+
+            elif draw == 'glyphrun':
+                if QGlyphRun:
+                    glyphrun = QGlyphRun()
+                    glyphrun.setRawFont(rawfont)
+                    glyphrun.setGlyphIndexes((glyph,))
+                    glyphrun.setPositions((QPointF(0, ascent),))
+                    painter.drawGlyphRun(QPointF(0, 0), glyphrun)
+                else:
+                    warnings.warn("QGlyphRun is unavailable for the current Qt binding! "
+                                  "QtAwesome will use the default draw values")
+                    return False
+            elif draw == 'image':
+                image = rawfont.alphaMapForGlyph(glyph, QRawFont.PixelAntialiasing) \
+                               .convertToFormat(QImage.Format_ARGB32_Premultiplied)
+                painter2 = QPainter(image)
+                painter2.setCompositionMode(QPainter.CompositionMode_SourceIn)
+                painter2.fillRect(image.rect(), painter.pen().color())
+                painter2.end()
+                brect = rawfont.boundingRect(glyph)
+                brect.translate(0, ascent)
+                painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
+                painter.drawImage(brect.topLeft(), image)
+
+            else:
+                # fallback to draw text if unknown value
+                return False
+
+            return True
+
+        if draw == 'text' or not try_draw_rawfont():
+            font = iconic.font(prefix, draw_size)
+            # Disable font hinting to mitigate tremulous spinning to some extent
+            # See spyder-ide/qtawesome#39
+            if animation is not None:
+                font.setHintingPreference(QFont.PreferNoHinting)
+            painter.setFont(font)
+            painter.drawText(rect, int(Qt.AlignCenter | Qt.AlignVCenter), char)
+
         painter.restore()
 
 
@@ -201,22 +321,26 @@ class IconicFont(QObject):
             - The ttf font filename,
             - The json charmap filename,
             - Optionally, the directory containing these files. When not
-              provided, the files will be looked for in ``./fonts/``.
+              provided, the files will be looked for in the QtAwesome ``fonts``
+              directory.
         """
         super().__init__()
         self.painter = CharIconPainter()
         self.painters = {}
         self.fontname = {}
+        self.fontdata = {}
         self.fontids = {}
         self.charmap = {}
         self.icon_cache = {}
+        self.rawfont_cache = {}
         for fargs in args:
             self.load_font(*fargs)
 
     def load_font(self, prefix, ttf_filename, charmap_filename, directory=None):
         """Loads a font file and the associated charmap.
 
-        If ``directory`` is None, the files will be looked for in ``./fonts/``.
+        If ``directory`` is None, the files will be looked for in
+        the qtawesome ``fonts`` directory.
 
         Parameters
         ----------
@@ -227,7 +351,7 @@ class IconicFont(QObject):
         charmap_filename: str
             Charmap filename
         directory: str or None, optional
-            Directory for font and charmap files
+            Directory path for font and charmap files
         """
 
         def hook(obj):
@@ -246,13 +370,13 @@ class IconicFont(QObject):
             return result
 
         if directory is None:
-            directory = os.path.join(
-                os.path.dirname(os.path.realpath(__file__)), 'fonts')
+            directory = self._get_fonts_directory()
 
         # Load font
         if QApplication.instance() is not None:
             with open(os.path.join(directory, ttf_filename), 'rb') as font_data:
-                id_ = QFontDatabase.addApplicationFontFromData(QByteArray(font_data.read()))
+                data = font_data.read()
+                id_ = QFontDatabase.addApplicationFontFromData(data)
             font_data.close()
 
             loadedFontFamilies = QFontDatabase.applicationFontFamilies(id_)
@@ -260,6 +384,7 @@ class IconicFont(QObject):
             if loadedFontFamilies:
                 self.fontids[prefix] = id_
                 self.fontname[prefix] = loadedFontFamilies[0]
+                self.fontdata[prefix] = data
             else:
                 raise FontError(u"Font at '{0}' appears to be empty. "
                                 "If you are on Windows 10, please read "
@@ -272,23 +397,19 @@ class IconicFont(QObject):
             with open(os.path.join(directory, charmap_filename), 'r') as codes:
                 self.charmap[prefix] = json.load(codes, object_hook=hook)
 
-            # Verify that vendorized fonts are not corrupt
-            if not SYSTEM_FONTS:
-                ttf_hash = MD5_HASHES.get(ttf_filename, None)
-                if ttf_hash is not None:
-                    hasher = hashlib.md5()
-                    with open(os.path.join(directory, ttf_filename),
-                              'rb') as f:
-                        content = f.read()
-                        hasher.update(content)
-                    ttf_calculated_hash_code = hasher.hexdigest()
-                    if ttf_calculated_hash_code != ttf_hash:
-                        raise FontError(u"Font is corrupt at: '{0}'".format(
-                                        os.path.join(directory, ttf_filename)))
-
     def icon(self, *names, **kwargs):
         """Return a QIcon object corresponding to the provided icon name."""
         cache_key = '{}{}'.format(names,kwargs)
+
+        if names and 'fa.' in names[0]:
+            warnings.warn(
+                "The FontAwesome 4.7 ('fa' prefix) icon set will be "
+                "removed in a future release in favor of FontAwesome 6. "
+                "We recommend you to move to FontAwesome 5 ('fa5*' prefix) "
+                "to prevent any issues in the future",
+                DeprecationWarning
+            )
+
         if cache_key not in self.icon_cache:
             options_list = kwargs.pop('options', [{}] * len(names))
             general_options = kwargs
@@ -316,7 +437,9 @@ class IconicFont(QObject):
         return self.icon_cache[cache_key]
 
     def _parse_options(self, specific_options, general_options, name):
-        options = dict(_default_options, **general_options)
+        live_dict = {k: v() if callable(v) else v for k, v in _default_options.items()}
+
+        options = dict(live_dict, **general_options)
         options.update(specific_options)
 
         # Handle icons for modes (Active, Disabled, Selected, Normal)
@@ -393,12 +516,31 @@ class IconicFont(QObject):
 
     def font(self, prefix, size):
         """Return a QFont corresponding to the given prefix and size."""
-        font = QFont(self.fontname[prefix])
+        font = QFont()
+        font.setFamily(self.fontname[prefix])
         font.setPixelSize(round(size))
         if prefix[-1] == 's':  # solid style
             font.setStyleName('Solid')
         return font
 
+    def rawfont(self, prefix, size, hintingPreference=QFont.PreferDefaultHinting):
+        """Return a QRawFont corresponding to the given prefix and size."""
+        cache = self.rawfont_cache
+        # https://doc.qt.io/qt-5/qrawfont.html
+        # QRawFont is considered local to the thread in which it is constructed
+        # (either using a constructor, or by calling loadFromData() or loadFromFile()).
+        # The QRawFont cannot be moved to a different thread,
+        # but will have to be recreated in the thread in question.
+        tid = str(QThread.currentThread())
+        if tid not in cache:
+            cache[tid] = {}
+            def clear_cache(): cache.pop(tid)
+            QThread().currentThread().finished.connect(clear_cache)
+        key = prefix, size, hintingPreference
+        if key not in cache[tid]:
+            cache[tid][key] = QRawFont(self.fontdata[prefix], size, hintingPreference)
+        return cache[tid][key]
+
     def set_custom_icon(self, name, painter):
         """Associate a user-provided CharIconPainter to an icon name.
 
@@ -428,3 +570,88 @@ class IconicFont(QObject):
         """Return the icon corresponding to the given painter."""
         engine = CharIconEngine(self, painter, options)
         return QIcon(engine)
+
+    def _get_fonts_directory(self):
+        """
+        Get bundled fonts directory.
+
+        On Windows an attempt to install the fonts per user is done
+        to prevent errors with fonts loading.
+
+        See spyder-ide/qtawesome#167 and spyder-ide/spyder#18642 for
+        context.
+        """
+        fonts_directory = os.path.join(
+            os.path.dirname(os.path.realpath(__file__)), 'fonts')
+        if os.name == 'nt':
+            fonts_directory = self._install_fonts(fonts_directory)
+        return fonts_directory
+
+    def _install_fonts(self, fonts_directory):
+        """
+        Copy the fonts to the user Fonts folder.
+        
+        Based on https://stackoverflow.com/a/41841088/15954282
+        """
+        # Try to get LOCALAPPDATA path
+        local_appdata_dir = os.environ.get('LOCALAPPDATA', None)
+        if not local_appdata_dir:
+            return fonts_directory
+
+        # Construct path to fonts from LOCALAPPDATA
+        user_fonts_dir = os.path.join(
+            local_appdata_dir, 'Microsoft', 'Windows', 'Fonts')
+        os.makedirs(user_fonts_dir, exist_ok=True)
+
+        # Setup bundled fonts on the LOCALAPPDATA fonts directory
+        for root, __, files in os.walk(fonts_directory):
+            for name in files:
+                src_path = os.path.join(root, name)
+                dst_path = os.path.join(
+                    user_fonts_dir,
+                    os.path.basename(src_path))
+
+                # Check if font already exists and copy font
+                if os.path.isfile(dst_path) and filecmp.cmp(src_path, dst_path):
+                    continue
+                shutil.copy(src_path, user_fonts_dir)
+
+                # Further process the font file (`.ttf`)
+                if os.path.splitext(name)[-1] == '.ttf':
+                    # Load the font in the current session
+                    if not gdi32.AddFontResourceW(dst_path):
+                        os.remove(dst_path)
+                        raise WindowsError(
+                            f'AddFontResource failed to load "{src_path}"')
+
+                    # Store the fontname/filename in the registry
+                    filename = os.path.basename(dst_path)
+                    fontname = os.path.splitext(filename)[0]
+
+                    # Try to get the font's real name
+                    cb = wintypes.DWORD()
+                    if gdi32.GetFontResourceInfoW(
+                            filename, ctypes.byref(cb), None, GFRI_DESCRIPTION):
+                        buf = (ctypes.c_wchar * cb.value)()
+                        if gdi32.GetFontResourceInfoW(
+                                filename, ctypes.byref(cb), buf, GFRI_DESCRIPTION):
+                            fontname = buf.value
+                    is_truetype = wintypes.BOOL()
+                    cb.value = ctypes.sizeof(is_truetype)
+                    gdi32.GetFontResourceInfoW(
+                        filename, ctypes.byref(cb), ctypes.byref(is_truetype),
+                        GFRI_ISTRUETYPE)
+                    if is_truetype:
+                        fontname += ' (TrueType)'
+                    try:
+                        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, FONTS_REG_PATH, 0,
+                                            winreg.KEY_SET_VALUE) as key:
+                            winreg.SetValueEx(key, fontname, 0, winreg.REG_SZ, filename)
+                    except OSError:
+                        # Needed to support older Windows version where
+                        # font installation per user is not possible/related registry
+                        # entry is not available
+                        # See spyder-ide/qtawesome#214 
+                        return fonts_directory
+
+        return user_fonts_dir
